@@ -1,13 +1,18 @@
 # flake8: noqa
 from django.forms import (BaseModelForm, model_to_dict, fields_for_model,
-                          save_instance, ValidationError, Media,
-                          MediaDefiningClass)
+                          ValidationError, Media, MediaDefiningClass)
 
 from .fields import *
 from .forms import *
 from .models import *
 from .widgets import *
 
+try:
+    # Django < 1.9
+    from django.forms import save_instance
+except ImportError:
+    pass
+
 try:
     from . import gis
 except Exception:
@@ -15,4 +20,4 @@ except Exception:
     warnings.warn(
         "Unable to import floppyforms.gis, geometry widgets not available")
 
-__version__ = '1.3.0'
+__version__ = '1.6.0'
