@@ -4,7 +4,7 @@ from django.forms import models
 
 from .fields import Field
 from .forms import LayoutRenderer
-from .widgets import Select, SelectMultiple
+from .widgets import Select, SelectMultiple, MultipleHiddenInput
 
 __all__ = ('ModelForm', 'ModelChoiceField', 'ModelMultipleChoiceField')
 
@@ -14,7 +14,7 @@ class ModelForm(LayoutRenderer, models.ModelForm):
         url = 'https://github.com/gregmuellegger/django-floppyforms/tree/1.2.0/CHANGES.rst'
         warnings.warn(
             'The behaviour of subclasses of floppyforms.models.ModelForm will '
-            'change with django-floppyforms 1.4. '
+            'change with django-floppyforms 2.0. '
             'Use `import floppyforms.__future__ as forms` instead of '
             '`import floppyforms as forms` to use the new behaviour now. '
             'See announcement here: %s' % url,
@@ -28,3 +28,4 @@ class ModelChoiceField(Field, models.ModelChoiceField):
 
 class ModelMultipleChoiceField(Field, models.ModelMultipleChoiceField):
     widget = SelectMultiple
+    hidden_widget = MultipleHiddenInput
