@@ -12,12 +12,14 @@ except ImportError:
 from django.forms.widgets import FILE_INPUT_CONTRADICTION
 from django.conf import settings
 from django.template import loader
-from django.utils.datastructures import MultiValueDict, MergeDict
 from django.utils.html import conditional_escape
 from django.utils.translation import ugettext_lazy as _
 from django.utils import datetime_safe, formats, six
 from django.utils.dates import MONTHS
 from django.utils.encoding import force_text
+from django.utils.safestring import mark_safe
+
+from .compat import MULTIVALUE_DICT_TYPES, flatten_contexts
 
 
 RE_DATE = re.compile(r'(\d{4})-(\d\d?)-(\d\d?)$')
@@ -57,6 +59,8 @@ class Input(Widget):
         if template_name is not None:
             self.template_name = template_name
         super(Input, self).__init__(*args, **kwargs)
+        # This attribute is used to inject a surrounding context in the
+        # floppyforms templatetags, when rendered inside a complete form.
         self.context_instance = None
 
     def get_context_data(self):
@@ -107,13 +111,12 @@ class Input(Widget):
         if template_name is None:
             template_name = self.template_name
         context = self.get_context(name, value, attrs=attrs or {}, **kwargs)
-        return loader.render_to_string(
-            template_name,
-            dictionary=context,
-            context_instance=self.context_instance)
+        context = flatten_contexts(self.context_instance, context)
+        return loader.render_to_string(template_name, context)
 
 
 class TextInput(Input):
+    template_name = 'floppyforms/text.html'
     input_type = 'text'
 
     def __init__(self, *args, **kwargs):
@@ -123,6 +126,7 @@ class TextInput(Input):
 
 
 class PasswordInput(TextInput):
+    template_name = 'floppyforms/password.html'
     input_type = 'password'
 
     def __init__(self, attrs=None, render_value=False):
@@ -136,6 +140,7 @@ class PasswordInput(TextInput):
 
 
 class HiddenInput(Input):
+    template_name = 'floppyforms/hidden.html'
     input_type = 'hidden'
 
 
@@ -159,15 +164,17 @@ class MultipleHiddenInput(HiddenInput):
             input_ = HiddenInput()
             input_.is_required = self.is_required
             inputs.append(input_.render(name, force_text(v), input_attrs))
-        return "\n".join(inputs)
+        return mark_safe("\n".join(inputs))
 
     def value_from_datadict(self, data, files, name):
-        if isinstance(data, (MultiValueDict, MergeDict)):
+        if isinstance(data, MULTIVALUE_DICT_TYPES):
             return data.getlist(name)
         return data.get(name, None)
 
 
 class SlugInput(TextInput):
+    template_name = 'floppyforms/slug.html'
+
     """<input type="text"> validating slugs with a pattern"""
     def get_context(self, name, value, attrs):
         context = super(SlugInput, self).get_context(name, value, attrs)
@@ -176,6 +183,8 @@ class SlugInput(TextInput):
 
 
 class IPAddressInput(TextInput):
+    template_name = 'floppyforms/ipaddress.html'
+
     """<input type="text"> validating IP addresses with a pattern"""
     ip_pattern = ("(25[0-5]|2[0-4]\d|[0-1]?\d?\d)(\.(25"
                   "[0-5]|2[0-4]\d|[0-1]?\d?\d)){3}")
@@ -187,6 +196,7 @@ class IPAddressInput(TextInput):
 
 
 class FileInput(Input):
+    template_name = 'floppyforms/file.html'
     input_type = 'file'
     needs_multipart_form = True
     omit_value = True
@@ -266,11 +276,15 @@ class Textarea(Input):
 
 
 class DateInput(Input):
+    template_name = 'floppyforms/date.html'
     input_type = 'date'
     supports_microseconds = False
 
     def __init__(self, attrs=None, format=None):
         super(DateInput, self).__init__(attrs)
+        # We hardcode the format here as the HTML5 input type="date" only
+        # accepts the ISO date format. See issue #115 for details:
+        # https://github.com/gregmuellegger/django-floppyforms/issues/115
         self.format = '%Y-%m-%d'
 
     def _format_value(self, value):
@@ -292,6 +306,7 @@ class DateInput(Input):
 
 
 class DateTimeInput(Input):
+    template_name = 'floppyforms/datetime.html'
     input_type = 'datetime'
     supports_microseconds = False
 
@@ -323,6 +338,7 @@ class DateTimeInput(Input):
 
 
 class TimeInput(Input):
+    template_name = 'floppyforms/time.html'
     input_type = 'time'
     supports_microseconds = False
 
@@ -353,22 +369,27 @@ class TimeInput(Input):
 
 
 class SearchInput(Input):
+    template_name = 'floppyforms/search.html'
     input_type = 'search'
 
 
 class EmailInput(TextInput):
+    template_name = 'floppyforms/email.html'
     input_type = 'email'
 
 
 class URLInput(TextInput):
+    template_name = 'floppyforms/url.html'
     input_type = 'url'
 
 
 class ColorInput(Input):
+    template_name = 'floppyforms/color.html'
     input_type = 'color'
 
 
 class NumberInput(TextInput):
+    template_name = 'floppyforms/number.html'
     input_type = 'number'
     min = None
     max = None
@@ -386,10 +407,12 @@ class NumberInput(TextInput):
 
 
 class RangeInput(NumberInput):
+    template_name = 'floppyforms/range.html'
     input_type = 'range'
 
 
 class PhoneNumberInput(Input):
+    template_name = 'floppyforms/phonenumber.html'
     input_type = 'tel'
 
 
@@ -398,6 +421,7 @@ def boolean_check(v):
 
 
 class CheckboxInput(Input, forms.CheckboxInput):
+    template_name = 'floppyforms/checkbox.html'
     input_type = 'checkbox'
 
     def __init__(self, attrs=None, check_test=None):
@@ -423,7 +447,7 @@ class CheckboxInput(Input, forms.CheckboxInput):
             return False
         value = data.get(name)
         values = {'true': True, 'false': False}
-        if isinstance(value, six.text_type):
+        if isinstance(value, six.string_types):
             value = values.get(value.lower(), value)
         return value
 
@@ -487,25 +511,25 @@ class Select(Input):
 
 class NullBooleanSelect(Select):
     def __init__(self, attrs=None):
-        choices = ((u'1', _('Unknown')),
-                   (u'2', _('Yes')),
-                   (u'3', _('No')))
+        choices = (('1', _('Unknown')),
+                   ('2', _('Yes')),
+                   ('3', _('No')))
         super(NullBooleanSelect, self).__init__(attrs, choices)
 
     def _format_value(self, value):
         value = value[0]
         try:
-            value = {True: u'2', False: u'3', u'2': u'2', u'3': u'3'}[value]
+            value = {True: '2', False: '3', '2': '2', '3': '3'}[value]
         except KeyError:
-            value = u'1'
+            value = '1'
         return value
 
     def value_from_datadict(self, data, files, name):
         value = data.get(name, None)
-        return {u'2': True,
+        return {'2': True,
                 True: True,
                 'True': True,
-                u'3': False,
+                '3': False,
                 'False': False,
                 False: False}.get(value, None)
 
@@ -527,7 +551,7 @@ class SelectMultiple(Select):
         return [force_text(v) for v in value]
 
     def value_from_datadict(self, data, files, name):
-        if isinstance(data, (MultiValueDict, MergeDict)):
+        if isinstance(data, MULTIVALUE_DICT_TYPES):
             return data.getlist(name)
         return data.get(name, None)
 
@@ -641,7 +665,7 @@ class SelectDateWidget(forms.Widget):
             year_val, month_val, day_val = value.year, value.month, value.day
         except AttributeError:
             year_val = month_val = day_val = None
-            if isinstance(value, six.text_type):
+            if isinstance(value, six.string_types):
                 if settings.USE_L10N:
                     try:
                         input_format = formats.get_format(
